@@ -28,6 +28,7 @@
 import __main__
 import abc
 import glob
+import itertools
 import keyword
 import logging
 import os
@@ -38,15 +39,10 @@ import builtins
 from enum import Enum
 from typing import (
     Any,
-    cast,
-    Dict,
-    Iterator,
-    List,
     Optional,
-    Set,
-    Tuple,
-    Sequence,
 )
+from collections.abc import Iterator, Sequence
+
 from . import inspection
 from . import line as lineparts
 from .line import LinePart
@@ -55,6 +51,9 @@ from .simpleeval import safe_eval, evaluate_current_expression, EvaluationError
 from .importcompletion import ModuleGatherer
 
 
+logger = logging.getLogger(__name__)
+
+
 # Autocomplete modes
 class AutocompleteModes(Enum):
     NONE = "none"
@@ -63,7 +62,7 @@ class AutocompleteModes(Enum):
     FUZZY = "fuzzy"
 
     @classmethod
-    def from_string(cls, value: str) -> Optional[Any]:
+    def from_string(cls, value: str) -> Optional["AutocompleteModes"]:
         if value.upper() in cls.__members__:
             return cls.__members__[value.upper()]
         return None
@@ -176,11 +175,11 @@ MAGIC_METHODS = tuple(
 KEYWORDS = frozenset(keyword.kwlist)
 
 
-def after_last_dot(name: str) -> str:
+def _after_last_dot(name: str) -> str:
     return name.rstrip(".").rsplit(".")[-1]
 
 
-def few_enough_underscores(current: str, match: str) -> bool:
+def _few_enough_underscores(current: str, match: str) -> bool:
     """Returns whether match should be shown based on current
 
     if current is _, True if match starts with 0 or 1 underscore
@@ -233,7 +232,7 @@ class BaseCompletionType:
     @abc.abstractmethod
     def matches(
         self, cursor_offset: int, line: str, **kwargs: Any
-    ) -> Optional[Set[str]]:
+    ) -> set[str] | None:
         """Returns a list of possible matches given a line and cursor, or None
         if this completion type isn't applicable.
 
@@ -252,7 +251,7 @@ class BaseCompletionType:
         raise NotImplementedError
 
     @abc.abstractmethod
-    def locate(self, cursor_offset: int, line: str) -> Optional[LinePart]:
+    def locate(self, cursor_offset: int, line: str) -> LinePart | None:
         """Returns a Linepart namedtuple instance or None given cursor and line
 
         A Linepart namedtuple contains a start, stop, and word. None is
@@ -265,7 +264,7 @@ class BaseCompletionType:
 
     def substitute(
         self, cursor_offset: int, line: str, match: str
-    ) -> Tuple[int, str]:
+    ) -> tuple[int, str]:
         """Returns a cursor offset and line with match swapped in"""
         lpart = self.locate(cursor_offset, line)
         assert lpart
@@ -296,7 +295,7 @@ class CumulativeCompleter(BaseCompletionType):
 
         super().__init__(True, mode)
 
-    def locate(self, cursor_offset: int, line: str) -> Optional[LinePart]:
+    def locate(self, cursor_offset: int, line: str) -> LinePart | None:
         for completer in self._completers:
             return_value = completer.locate(cursor_offset, line)
             if return_value is not None:
@@ -308,7 +307,7 @@ class CumulativeCompleter(BaseCompletionType):
 
     def matches(
         self, cursor_offset: int, line: str, **kwargs: Any
-    ) -> Optional[Set]:
+    ) -> set[str] | None:
         return_value = None
         all_matches = set()
         for completer in self._completers:
@@ -333,33 +332,34 @@ class ImportCompletion(BaseCompletionType):
 
     def matches(
         self, cursor_offset: int, line: str, **kwargs: Any
-    ) -> Optional[Set]:
+    ) -> set[str] | None:
         return self.module_gatherer.complete(cursor_offset, line)
 
-    def locate(self, cursor_offset: int, line: str) -> Optional[LinePart]:
+    def locate(self, cursor_offset: int, line: str) -> LinePart | None:
         return lineparts.current_word(cursor_offset, line)
 
     def format(self, word: str) -> str:
-        return after_last_dot(word)
+        return _after_last_dot(word)
+
+
+def _safe_glob(pathname: str) -> Iterator[str]:
+    return glob.iglob(glob.escape(pathname) + "*")
 
 
 class FilenameCompletion(BaseCompletionType):
     def __init__(self, mode: AutocompleteModes = AutocompleteModes.SIMPLE):
         super().__init__(False, mode)
 
-    def safe_glob(self, pathname: str) -> Iterator[str]:
-        return glob.iglob(glob.escape(pathname) + "*")
-
     def matches(
         self, cursor_offset: int, line: str, **kwargs: Any
-    ) -> Optional[Set]:
+    ) -> set[str] | None:
         cs = lineparts.current_string(cursor_offset, line)
         if cs is None:
             return None
         matches = set()
         username = cs.word.split(os.path.sep, 1)[0]
         user_dir = os.path.expanduser(username)
-        for filename in self.safe_glob(os.path.expanduser(cs.word)):
+        for filename in _safe_glob(os.path.expanduser(cs.word)):
             if os.path.isdir(filename):
                 filename += os.path.sep
             if cs.word.startswith("~"):
@@ -367,11 +367,10 @@ class FilenameCompletion(BaseCompletionType):
             matches.add(filename)
         return matches
 
-    def locate(self, cursor_offset: int, line: str) -> Optional[LinePart]:
+    def locate(self, cursor_offset: int, line: str) -> LinePart | None:
         return lineparts.current_string(cursor_offset, line)
 
     def format(self, filename: str) -> str:
-        filename.rstrip(os.sep).rsplit(os.sep)[-1]
         if os.sep in filename[:-1]:
             return filename[filename.rindex(os.sep, 0, -1) + 1 :]
         else:
@@ -379,16 +378,16 @@ class FilenameCompletion(BaseCompletionType):
 
 
 class AttrCompletion(BaseCompletionType):
-
     attr_matches_re = LazyReCompile(r"(\w+(\.\w+)*)\.(\w*)")
 
     def matches(
-        self, cursor_offset: int, line: str, **kwargs: Any
-    ) -> Optional[Set]:
-        if "locals_" not in kwargs:
-            return None
-        locals_ = cast(Dict[str, Any], kwargs["locals_"])
-
+        self,
+        cursor_offset: int,
+        line: str,
+        *,
+        locals_: dict[str, Any] | None = None,
+        **kwargs: Any,
+    ) -> set[str] | None:
         r = self.locate(cursor_offset, line)
         if r is None:
             return None
@@ -398,48 +397,46 @@ class AttrCompletion(BaseCompletionType):
 
         assert "." in r.word
 
-        for i in range(1, len(r.word) + 1):
-            if r.word[-i] == "[":
-                i -= 1
-                break
-        methodtext = r.word[-i:]
+        i = r.word.rfind("[") + 1
+        methodtext = r.word[i:]
         matches = {
-            "".join([r.word[:-i], m])
+            "".join([r.word[:i], m])
             for m in self.attr_matches(methodtext, locals_)
         }
 
         return {
             m
             for m in matches
-            if few_enough_underscores(r.word.split(".")[-1], m.split(".")[-1])
+            if _few_enough_underscores(r.word.split(".")[-1], m.split(".")[-1])
         }
 
-    def locate(self, cursor_offset: int, line: str) -> Optional[LinePart]:
+    def locate(self, cursor_offset: int, line: str) -> LinePart | None:
         return lineparts.current_dotted_attribute(cursor_offset, line)
 
     def format(self, word: str) -> str:
-        return after_last_dot(word)
+        return _after_last_dot(word)
 
-    def attr_matches(self, text: str, namespace: Dict[str, Any]) -> List:
+    def attr_matches(
+        self, text: str, namespace: dict[str, Any]
+    ) -> Iterator[str]:
         """Taken from rlcompleter.py and bent to my will."""
 
         m = self.attr_matches_re.match(text)
         if not m:
-            return []
+            return (_ for _ in ())
 
         expr, attr = m.group(1, 3)
         if expr.isdigit():
             # Special case: float literal, using attrs here will result in
             # a SyntaxError
-            return []
+            return (_ for _ in ())
         try:
             obj = safe_eval(expr, namespace)
         except EvaluationError:
-            return []
-        matches = self.attr_lookup(obj, expr, attr)
-        return matches
+            return (_ for _ in ())
+        return self.attr_lookup(obj, expr, attr)
 
-    def attr_lookup(self, obj: Any, expr: str, attr: str) -> List:
+    def attr_lookup(self, obj: Any, expr: str, attr: str) -> Iterator[str]:
         """Second half of attr_matches."""
         words = self.list_attributes(obj)
         if inspection.hasattr_safe(obj, "__class__"):
@@ -452,27 +449,32 @@ class AttrCompletion(BaseCompletionType):
                 except ValueError:
                     pass
 
-        matches = []
         n = len(attr)
-        for word in words:
-            if self.method_match(word, n, attr) and word != "__builtins__":
-                matches.append(f"{expr}.{word}")
-        return matches
+        return (
+            f"{expr}.{word}"
+            for word in words
+            if self.method_match(word, n, attr) and word != "__builtins__"
+        )
 
-    def list_attributes(self, obj: Any) -> List[str]:
-        # TODO: re-implement dir using getattr_static to avoid using
-        # AttrCleaner here?
+    def list_attributes(self, obj: Any) -> list[str]:
+        # TODO: re-implement dir without AttrCleaner here
+        #
+        # Note: accessing `obj.__dir__` via `getattr_static` is not side-effect free.
         with inspection.AttrCleaner(obj):
             return dir(obj)
 
 
 class DictKeyCompletion(BaseCompletionType):
     def matches(
-        self, cursor_offset: int, line: str, **kwargs: Any
-    ) -> Optional[Set]:
-        if "locals_" not in kwargs:
+        self,
+        cursor_offset: int,
+        line: str,
+        *,
+        locals_: dict[str, Any] | None = None,
+        **kwargs: Any,
+    ) -> set[str] | None:
+        if locals_ is None:
             return None
-        locals_ = kwargs["locals_"]
 
         r = self.locate(cursor_offset, line)
         if r is None:
@@ -481,7 +483,7 @@ class DictKeyCompletion(BaseCompletionType):
         if current_dict_parts is None:
             return None
 
-        _, _, dexpr = current_dict_parts
+        dexpr = current_dict_parts.word
         try:
             obj = safe_eval(dexpr, locals_)
         except EvaluationError:
@@ -494,7 +496,7 @@ class DictKeyCompletion(BaseCompletionType):
         else:
             return None
 
-    def locate(self, cursor_offset: int, line: str) -> Optional[LinePart]:
+    def locate(self, cursor_offset: int, line: str) -> LinePart | None:
         return lineparts.current_dict_key(cursor_offset, line)
 
     def format(self, match: str) -> str:
@@ -503,11 +505,20 @@ class DictKeyCompletion(BaseCompletionType):
 
 class MagicMethodCompletion(BaseCompletionType):
     def matches(
-        self, cursor_offset: int, line: str, **kwargs: Any
-    ) -> Optional[Set]:
-        if "current_block" not in kwargs:
+        self,
+        cursor_offset: int,
+        line: str,
+        *,
+        current_block: str | None = None,
+        complete_magic_methods: bool | None = None,
+        **kwargs: Any,
+    ) -> set[str] | None:
+        if (
+            current_block is None
+            or complete_magic_methods is None
+            or not complete_magic_methods
+        ):
             return None
-        current_block = kwargs["current_block"]
 
         r = self.locate(cursor_offset, line)
         if r is None:
@@ -516,31 +527,34 @@ class MagicMethodCompletion(BaseCompletionType):
             return None
         return {name for name in MAGIC_METHODS if name.startswith(r.word)}
 
-    def locate(self, cursor_offset: int, line: str) -> Optional[LinePart]:
+    def locate(self, cursor_offset: int, line: str) -> LinePart | None:
         return lineparts.current_method_definition_name(cursor_offset, line)
 
 
 class GlobalCompletion(BaseCompletionType):
     def matches(
-        self, cursor_offset: int, line: str, **kwargs: Any
-    ) -> Optional[Set]:
+        self,
+        cursor_offset: int,
+        line: str,
+        *,
+        locals_: dict[str, Any] | None = None,
+        **kwargs: Any,
+    ) -> set[str] | None:
         """Compute matches when text is a simple name.
         Return a list of all keywords, built-in functions and names currently
         defined in self.namespace that match.
         """
-        if "locals_" not in kwargs:
+        if locals_ is None:
             return None
-        locals_ = kwargs["locals_"]
 
         r = self.locate(cursor_offset, line)
         if r is None:
             return None
 
-        matches = set()
         n = len(r.word)
-        for word in KEYWORDS:
-            if self.method_match(word, n, r.word):
-                matches.add(word)
+        matches = {
+            word for word in KEYWORDS if self.method_match(word, n, r.word)
+        }
         for nspace in (builtins.__dict__, locals_):
             for word, val in nspace.items():
                 # if identifier isn't ascii, don't complete (syntax error)
@@ -553,50 +567,60 @@ class GlobalCompletion(BaseCompletionType):
                     matches.add(_callable_postfix(val, word))
         return matches if matches else None
 
-    def locate(self, cursor_offset: int, line: str) -> Optional[LinePart]:
+    def locate(self, cursor_offset: int, line: str) -> LinePart | None:
         return lineparts.current_single_word(cursor_offset, line)
 
 
 class ParameterNameCompletion(BaseCompletionType):
     def matches(
-        self, cursor_offset: int, line: str, **kwargs: Any
-    ) -> Optional[Set]:
-        if "argspec" not in kwargs:
+        self,
+        cursor_offset: int,
+        line: str,
+        *,
+        funcprops: inspection.FuncProps | None = None,
+        **kwargs: Any,
+    ) -> set[str] | None:
+        if funcprops is None:
             return None
-        argspec = kwargs["argspec"]
 
-        if not argspec:
-            return None
         r = self.locate(cursor_offset, line)
         if r is None:
             return None
-        if argspec:
-            matches = {
-                f"{name}="
-                for name in argspec[1][0]
-                if isinstance(name, str) and name.startswith(r.word)
-            }
-            matches.update(
-                name + "=" for name in argspec[1][4] if name.startswith(r.word)
-            )
+
+        matches = {
+            f"{name}="
+            for name in funcprops.argspec.args
+            if isinstance(name, str) and name.startswith(r.word)
+        }
+        matches.update(
+            f"{name}="
+            for name in funcprops.argspec.kwonly
+            if name.startswith(r.word)
+        )
         return matches if matches else None
 
-    def locate(self, cursor_offset: int, line: str) -> Optional[LinePart]:
-        return lineparts.current_word(cursor_offset, line)
+    def locate(self, cursor_offset: int, line: str) -> LinePart | None:
+        r = lineparts.current_word(cursor_offset, line)
+        if r and r.word[-1] == "(":
+            # if the word ends with a (, it's the parent word with an empty
+            # param. Return an empty word
+            return lineparts.LinePart(r.stop, r.stop, "")
+        return r
 
 
 class ExpressionAttributeCompletion(AttrCompletion):
     # could replace attr completion as a more general case with some work
-    def locate(self, cursor_offset: int, line: str) -> Optional[LinePart]:
+    def locate(self, cursor_offset: int, line: str) -> LinePart | None:
         return lineparts.current_expression_attribute(cursor_offset, line)
 
     def matches(
-        self, cursor_offset: int, line: str, **kwargs: Any
-    ) -> Optional[Set]:
-        if "locals_" not in kwargs:
-            return None
-        locals_ = kwargs["locals_"]
-
+        self,
+        cursor_offset: int,
+        line: str,
+        *,
+        locals_: dict[str, Any] | None = None,
+        **kwargs: Any,
+    ) -> set[str] | None:
         if locals_ is None:
             locals_ = __main__.__dict__
 
@@ -610,7 +634,7 @@ class ExpressionAttributeCompletion(AttrCompletion):
 
         # strips leading dot
         matches = (m[1:] for m in self.attr_lookup(obj, "", attr.word))
-        return {m for m in matches if few_enough_underscores(attr.word, m)}
+        return {m for m in matches if _few_enough_underscores(attr.word, m)}
 
 
 try:
@@ -620,33 +644,44 @@ except ImportError:
     class MultilineJediCompletion(BaseCompletionType):  # type: ignore [no-redef]
         def matches(
             self, cursor_offset: int, line: str, **kwargs: Any
-        ) -> Optional[Set]:
+        ) -> set[str] | None:
             return None
 
-        def locate(self, cursor_offset: int, line: str) -> Optional[LinePart]:
+        def locate(self, cursor_offset: int, line: str) -> LinePart | None:
             return None
 
-
 else:
 
-    class JediCompletion(BaseCompletionType):
-        _orig_start: Optional[int]
+    class MultilineJediCompletion(BaseCompletionType):  # type: ignore [no-redef]
+        _orig_start: int | None
 
         def matches(
-            self, cursor_offset: int, line: str, **kwargs: Any
-        ) -> Optional[Set]:
-            if "history" not in kwargs:
+            self,
+            cursor_offset: int,
+            line: str,
+            *,
+            current_block: str | None = None,
+            history: list[str] | None = None,
+            **kwargs: Any,
+        ) -> set[str] | None:
+            if (
+                current_block is None
+                or history is None
+                or "\n" not in current_block
+                or not lineparts.current_word(cursor_offset, line)
+            ):
                 return None
-            history = kwargs["history"]
 
-            if not lineparts.current_word(cursor_offset, line):
-                return None
-            history = "\n".join(history) + "\n" + line
+            assert cursor_offset <= len(line), "{!r} {!r}".format(
+                cursor_offset,
+                line,
+            )
 
+            combined_history = "\n".join(itertools.chain(history, (line,)))
             try:
-                script = jedi.Script(history, path="fake.py")
+                script = jedi.Script(combined_history, path="fake.py")
                 completions = script.complete(
-                    len(history.splitlines()), cursor_offset
+                    combined_history.count("\n") + 1, cursor_offset
                 )
             except (jedi.NotFoundError, IndexError, KeyError):
                 # IndexError for #483
@@ -662,8 +697,6 @@ else:
                 return None
             assert isinstance(self._orig_start, int)
 
-            first_letter = line[self._orig_start : self._orig_start + 1]
-
             matches = [c.name for c in completions]
             if any(
                 not m.lower().startswith(matches[0][0].lower()) for m in matches
@@ -673,40 +706,27 @@ else:
                 return None
             else:
                 # case-sensitive matches only
+                first_letter = line[self._orig_start]
                 return {m for m in matches if m.startswith(first_letter)}
 
         def locate(self, cursor_offset: int, line: str) -> LinePart:
-            assert isinstance(self._orig_start, int)
+            assert self._orig_start is not None
             start = self._orig_start
             end = cursor_offset
             return LinePart(start, end, line[start:end])
 
-    class MultilineJediCompletion(JediCompletion):  # type: ignore [no-redef]
-        def matches(
-            self, cursor_offset: int, line: str, **kwargs: Any
-        ) -> Optional[Set]:
-            if "current_block" not in kwargs or "history" not in kwargs:
-                return None
-            current_block = kwargs["current_block"]
-            history = kwargs["history"]
-
-            if "\n" in current_block:
-                assert cursor_offset <= len(line), "{!r} {!r}".format(
-                    cursor_offset,
-                    line,
-                )
-                results = super().matches(cursor_offset, line, history=history)
-                return results
-            else:
-                return None
-
 
 def get_completer(
     completers: Sequence[BaseCompletionType],
     cursor_offset: int,
     line: str,
-    **kwargs: Any,
-) -> Tuple[List[str], Optional[BaseCompletionType]]:
+    *,
+    locals_: dict[str, Any] | None = None,
+    argspec: inspection.FuncProps | None = None,
+    history: list[str] | None = None,
+    current_block: str | None = None,
+    complete_magic_methods: bool | None = None,
+) -> tuple[list[str], BaseCompletionType | None]:
     """Returns a list of matches and an applicable completer
 
     If no matches available, returns a tuple of an empty list and None
@@ -715,7 +735,7 @@ def get_completer(
     line is a string of the current line
     kwargs (all optional):
         locals_ is a dictionary of the environment
-        argspec is an inspect.ArgSpec instance for the current function where
+        argspec is an inspection.FuncProps instance for the current function where
             the cursor is
         current_block is the possibly multiline not-yet-evaluated block of
             code which the current line is part of
@@ -723,27 +743,44 @@ def get_completer(
             double underscore methods like __len__ in method signatures
     """
 
+    def _cmpl_sort(x: str) -> tuple[bool, str]:
+        """
+        Function used to sort the matches.
+        """
+        # put parameters above everything in completion
+        return (
+            x[-1] != "=",
+            x,
+        )
+
     for completer in completers:
         try:
-            matches = completer.matches(cursor_offset, line, **kwargs)
+            matches = completer.matches(
+                cursor_offset,
+                line,
+                locals_=locals_,
+                funcprops=argspec,
+                history=history,
+                current_block=current_block,
+                complete_magic_methods=complete_magic_methods,
+            )
         except Exception as e:
             # Instead of crashing the UI, log exceptions from autocompleters.
-            logger = logging.getLogger(__name__)
             logger.debug(
-                "Completer {} failed with unhandled exception: {}".format(
-                    completer, e
-                )
+                "Completer %r failed with unhandled exception: %s", completer, e
             )
             continue
         if matches is not None:
-            return sorted(matches), (completer if matches else None)
+            return sorted(matches, key=_cmpl_sort), (
+                completer if matches else None
+            )
 
     return [], None
 
 
 def get_default_completer(
     mode: AutocompleteModes, module_gatherer: ModuleGatherer
-) -> Tuple[BaseCompletionType, ...]:
+) -> tuple[BaseCompletionType, ...]:
     return (
         (
             DictKeyCompletion(mode=mode),
