@@ -19,54 +19,60 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
-#
 
 import string
+from typing import TypeVar, Generic
+
+T = TypeVar("T")
 
 
-class KeyMap:
+class KeyMap(Generic[T]):
+    def __init__(self, default: T) -> None:
+        self.map: dict[str, T] = {}
+        self.default = default
 
-    def __init__(self):
-        self.map = {}
-
-    def __getitem__(self, key):
+    def __getitem__(self, key: str) -> T:
         if not key:
             # Unbound key
-            return str()
+            return self.default
         elif key in self.map:
             return self.map[key]
         else:
-            raise KeyError('Configured keymap (%s)' % key +
-                           ' does not exist in bpython.keys')
+            raise KeyError(
+                f"Configured keymap ({key}) does not exist in bpython.keys"
+            )
 
-    def __delitem__(self, key):
+    def __delitem__(self, key: str) -> None:
         del self.map[key]
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key: str, value: T) -> None:
         self.map[key] = value
 
-cli_key_dispatch = KeyMap()
-urwid_key_dispatch = KeyMap()
+
+cli_key_dispatch: KeyMap[tuple[str, ...]] = KeyMap(tuple())
+urwid_key_dispatch = KeyMap("")
 
 # fill dispatch with letters
 for c in string.ascii_lowercase:
-    cli_key_dispatch['C-%s' % c] = (chr(string.ascii_lowercase.index(c) + 1),
-                                '^%s' % c.upper())
+    cli_key_dispatch[f"C-{c}"] = (
+        chr(string.ascii_lowercase.index(c) + 1),
+        f"^{c.upper()}",
+    )
 
 for c in string.ascii_lowercase:
-    urwid_key_dispatch['C-%s' % c] = 'ctrl %s' % c
-    urwid_key_dispatch['M-%s' % c] = 'meta %s' % c
+    urwid_key_dispatch[f"C-{c}"] = f"ctrl {c}"
+    urwid_key_dispatch[f"M-{c}"] = f"meta {c}"
 
 # fill dispatch with cool characters
-cli_key_dispatch['C-['] = (chr(27), '^[')
-cli_key_dispatch['C-\\'] = (chr(28), '^\\')
-cli_key_dispatch['C-]'] = (chr(29), '^]')
-cli_key_dispatch['C-^'] = (chr(30), '^^')
-cli_key_dispatch['C-_'] = (chr(31), '^_')
+cli_key_dispatch["C-["] = (chr(27), "^[")
+cli_key_dispatch["C-\\"] = (chr(28), "^\\")
+cli_key_dispatch["C-]"] = (chr(29), "^]")
+cli_key_dispatch["C-^"] = (chr(30), "^^")
+cli_key_dispatch["C-_"] = (chr(31), "^_")
 
 # fill dispatch with function keys
-for x in xrange(1, 13):
-    cli_key_dispatch['F%s' % str(x)] = ('KEY_F(%s)' % str(x),)
+for x in range(1, 13):
+    cli_key_dispatch[f"F{x}"] = (f"KEY_F({x})",)
 
-for x in xrange(1, 13):
-    urwid_key_dispatch['F%s' % str(x)] = 'f%s' % str(x)
+for x in range(1, 13):
+    urwid_key_dispatch[f"F{x}"] = f"f{x}"
