@@ -26,26 +26,20 @@ In order to provide fancy completion, some code can be executed safely.
 """
 
 import ast
-import sys
 import builtins
-from typing import Dict, Any
+from typing import Any
 
 from . import line as line_properties
 from .inspection import getattr_safe
 
-_is_py38 = sys.version_info[:2] >= (3, 8)
-_is_py39 = sys.version_info[:2] >= (3, 9)
-
-_string_type_nodes = (ast.Str, ast.Bytes)
 _numeric_types = (int, float, complex)
-_name_type_nodes = (ast.Name,) if _is_py38 else (ast.Name, ast.NameConstant)
 
 
 class EvaluationError(Exception):
     """Raised if an exception occurred in safe_eval."""
 
 
-def safe_eval(expr: str, namespace: Dict[str, Any]) -> Any:
+def safe_eval(expr: str, namespace: dict[str, Any]) -> Any:
     """Not all that safe, just catches some errors"""
     try:
         return eval(expr, namespace)
@@ -91,10 +85,6 @@ def simple_eval(node_or_string, namespace=None):
     def _convert(node):
         if isinstance(node, ast.Constant):
             return node.value
-        elif not _is_py38 and isinstance(node, _string_type_nodes):
-            return node.s
-        elif not _is_py38 and isinstance(node, ast.Num):
-            return node.n
         elif isinstance(node, ast.Tuple):
             return tuple(map(_convert, node.elts))
         elif isinstance(node, ast.List):
@@ -130,7 +120,7 @@ def simple_eval(node_or_string, namespace=None):
             return list()
 
         # this is a deviation from literal_eval: we allow non-literals
-        elif isinstance(node, _name_type_nodes):
+        elif isinstance(node, ast.Name):
             try:
                 return namespace[node.id]
             except KeyError:
@@ -154,7 +144,9 @@ def simple_eval(node_or_string, namespace=None):
         elif isinstance(node, ast.BinOp) and isinstance(
             node.op, (ast.Add, ast.Sub)
         ):
-            # ast.literal_eval does ast typechecks here, we use type checks
+            # this is a deviation from literal_eval: ast.literal_eval accepts
+            # (+/-) int, float and complex literals as left operand, and complex
+            # as right operation, we evaluate as much as possible
             left = _convert(node.left)
             right = _convert(node.right)
             if not (
@@ -168,18 +160,8 @@ def simple_eval(node_or_string, namespace=None):
                 return left - right
 
         # this is a deviation from literal_eval: we allow indexing
-        elif (
-            not _is_py39
-            and isinstance(node, ast.Subscript)
-            and isinstance(node.slice, ast.Index)
-        ):
-            obj = _convert(node.value)
-            index = _convert(node.slice.value)
-            return safe_getitem(obj, index)
-        elif (
-            _is_py39
-            and isinstance(node, ast.Subscript)
-            and isinstance(node.slice, (ast.Constant, ast.Name))
+        elif isinstance(node, ast.Subscript) and isinstance(
+            node.slice, (ast.Constant, ast.Name)
         ):
             obj = _convert(node.value)
             index = _convert(node.slice)
@@ -203,7 +185,7 @@ def safe_getitem(obj, index):
             return obj[index]
         except (KeyError, IndexError):
             raise EvaluationError(f"can't lookup key {index!r} on {obj!r}")
-    raise ValueError("unsafe to lookup on object of type {}".format(type(obj)))
+    raise ValueError(f"unsafe to lookup on object of type {type(obj)}")
 
 
 def find_attribute_with_name(node, name):
@@ -216,8 +198,8 @@ def find_attribute_with_name(node, name):
 
 
 def evaluate_current_expression(
-    cursor_offset: int, line: str, namespace: Dict[str, Any] = None
-):
+    cursor_offset: int, line: str, namespace: dict[str, Any] | None = None
+) -> Any:
     """
     Return evaluated expression to the right of the dot of current attribute.
 
@@ -227,9 +209,6 @@ def evaluate_current_expression(
     # Find the biggest valid ast.
     # Once our attribute access is found, return its .value subtree
 
-    if namespace is None:
-        namespace = {}
-
     # in case attribute is blank, e.g. foo.| -> foo.xxx|
     temp_line = line[:cursor_offset] + "xxx" + line[cursor_offset:]
     temp_cursor = cursor_offset + 3
