@@ -11,7 +11,8 @@ try:
 except ImportError:
     has_jedi = False
 
-from bpython import autocomplete
+from bpython import autocomplete, inspection
+from bpython.line import LinePart
 
 glob_function = "glob.iglob"
 
@@ -34,7 +35,7 @@ class TestFormatters(unittest.TestCase):
         self.assertEqual(last_part_of_filename("ab.c/e.f.g/"), "e.f.g/")
 
     def test_attribute(self):
-        self.assertEqual(autocomplete.after_last_dot("abc.edf"), "edf")
+        self.assertEqual(autocomplete._after_last_dot("abc.edf"), "edf")
 
 
 def completer(matches):
@@ -105,6 +106,15 @@ class TestCumulativeCompleter(unittest.TestCase):
         cumulative = autocomplete.CumulativeCompleter([a, b])
         self.assertEqual(cumulative.matches(3, "abc"), {"a", "b"})
 
+    def test_order_completer(self):
+        a = self.completer(["ax", "ab="])
+        b = self.completer(["aa"])
+        cumulative = autocomplete.CumulativeCompleter([a, b])
+        self.assertEqual(
+            autocomplete.get_completer([cumulative], 1, "a"),
+            (["ab=", "aa", "ax"], cumulative),
+        )
+
 
 class TestFilenameCompletion(unittest.TestCase):
     def setUp(self):
@@ -114,7 +124,9 @@ class TestFilenameCompletion(unittest.TestCase):
         self.assertEqual(self.completer.locate(4, "abcd"), None)
 
     def test_locate_succeeds_when_in_string(self):
-        self.assertEqual(self.completer.locate(4, "a'bc'd"), (2, 4, "bc"))
+        self.assertEqual(
+            self.completer.locate(4, "a'bc'd"), LinePart(2, 4, "bc")
+        )
 
     def test_issue_491(self):
         self.assertNotEqual(self.completer.matches(9, '"a[a.l-1]'), None)
@@ -321,12 +333,17 @@ class TestMagicMethodCompletion(unittest.TestCase):
         com = autocomplete.MagicMethodCompletion()
         block = "class Something(object)\n    def __"
         self.assertSetEqual(
-            com.matches(10, "    def __", current_block=block),
+            com.matches(
+                10,
+                "    def __",
+                current_block=block,
+                complete_magic_methods=True,
+            ),
             set(autocomplete.MAGIC_METHODS),
         )
 
 
-Comp = namedtuple("Completion", ["name", "complete"])
+Completion = namedtuple("Completion", ["name", "complete"])
 
 
 @unittest.skipUnless(has_jedi, "jedi required")
@@ -363,7 +380,7 @@ class TestMultilineJediCompletion(unittest.TestCase):
             " a",
             "class Foo:\n a",
             ["adsf"],
-            [Comp("Abc", "bc"), Comp("Cbc", "bc")],
+            [Completion("Abc", "bc"), Completion("Cbc", "bc")],
         )
         self.assertEqual(matches, None)
 
@@ -373,7 +390,7 @@ class TestMultilineJediCompletion(unittest.TestCase):
             " a",
             "class Foo:\n a",
             ["adsf"],
-            [Comp("Abc", "bc"), Comp("ade", "de")],
+            [Completion("Abc", "bc"), Completion("ade", "de")],
         )
         self.assertSetEqual(matches, {"ade"})
 
@@ -415,11 +432,19 @@ class TestParameterNameCompletion(unittest.TestCase):
         def func(apple, apricot, banana, carrot):
             pass
 
-        argspec = list(inspect.getfullargspec(func))
-        argspec = ["func", argspec, False]
+        argspec = inspection.ArgSpec(*inspect.getfullargspec(func))
+        funcspec = inspection.FuncProps("func", argspec, False)
         com = autocomplete.ParameterNameCompletion()
         self.assertSetEqual(
-            com.matches(1, "a", argspec=argspec), {"apple=", "apricot="}
+            com.matches(1, "a", funcprops=funcspec), {"apple=", "apricot="}
+        )
+        self.assertSetEqual(
+            com.matches(2, "ba", funcprops=funcspec), {"banana="}
+        )
+        self.assertSetEqual(
+            com.matches(3, "car", funcprops=funcspec), {"carrot="}
+        )
+        self.assertSetEqual(
+            com.matches(5, "func(", funcprops=funcspec),
+            {"apple=", "apricot=", "banana=", "carrot="},
         )
-        self.assertSetEqual(com.matches(2, "ba", argspec=argspec), {"banana="})
-        self.assertSetEqual(com.matches(3, "car", argspec=argspec), {"carrot="})
